package com.jyc.threegames.activity;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.text.Html;
import android.view.View;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.constraintlayout.widget.ConstraintLayout;

import com.jyc.threegames.R;
import com.jyc.threegames.activity.base.BaseActivity;
import com.jyc.threegames.bean.GameInfo;

import java.util.Locale;

import butterknife.BindView;

public class GameActivity extends BaseActivity {
    private static final String PARAM_GAME_INFO = "gameInfo";

    private static final int PRE_GAME_COUNT_DOWN = 5;
    private int mCurrentPreGameCountDown = 0;
    private Handler mPreGameHandler;

    private GameInfo mGameInfo;

    private int mSpecialNumberOne;
    private int mSpecialNumberTwo;

    @BindView(R.id.pre_game_hint)
    TextView mTVPreGameHint;

    @BindView(R.id.game_hint)
    TextView mTVGameHint;

    @BindView(R.id.number)
    TextView mTVNumber;

    @BindView(R.id.click_area)
    View mVClickArea;

    @Override
    protected void init(Bundle instance) {
        super.init(instance);
        if (instance == null)
            mGameInfo = getIntent().getParcelableExtra(PARAM_GAME_INFO);
        else
            mGameInfo = instance.getParcelable(PARAM_GAME_INFO);

        startPrePractise();
    }

    @Override
    protected void onSaveInstanceState(@NonNull Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putParcelable(PARAM_GAME_INFO, mGameInfo);
    }

    @Override
    protected void onRestoreInstanceState(@NonNull Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        mGameInfo = savedInstanceState.getParcelable(PARAM_GAME_INFO);
    }

    @Override
    protected int getRootLayout() {
        return R.layout.activity_game_one;
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
    }

    private void startPrePractise(){
        hideAllView();

        mSpecialNumberOne = getRandomNumber();
        mSpecialNumberTwo = getRandomNumber(mSpecialNumberOne);

        mCurrentPreGameCountDown = PRE_GAME_COUNT_DOWN;
        mPreGameHandler = new Handler(Looper.getMainLooper(), new Handler.Callback() {
            @Override
            public boolean handleMessage(@NonNull Message message) {
                if (mCurrentPreGameCountDown > 0){
                    mTVPreGameHint.setVisibility(View.VISIBLE);
                    mTVPreGameHint.setText(Html.fromHtml(getPrePractiseText(mSpecialNumberOne, mSpecialNumberTwo, mCurrentPreGameCountDown), Html.FROM_HTML_MODE_LEGACY));
                    mCurrentPreGameCountDown --;
                    mPreGameHandler.sendEmptyMessageDelayed(0, 1000);
                }else{
                    mTVPreGameHint.setVisibility(View.GONE);
                    startPractise();
                }
                return true;
            }
        });
        mPreGameHandler.sendEmptyMessage(0);
    }

    private String getPrePractiseText(int numberOne, int numberTwo, int countDown){
        switch (mGameInfo.gameVersion){
            case GameInfo.VERSION_GAME_ONE:
                return String.format(Locale.TRADITIONAL_CHINESE,"當屏幕出現數字<font color='#cc0000'>%1$d</font>的時候，<br/>請您以最快的速度按下<br/><br/>正式開始前，會有小段練習<br/><br/><font color='#cc0000'>%2$d</font>秒后進入練習", mSpecialNumberOne, countDown);
            case GameInfo.VERSION_GAME_TWO:
                return String.format(Locale.TRADITIONAL_CHINESE,"當數字<font color='#cc0000'>%1$d</font>,<font color='#cc0000'>%2$d</font>連續出現時，<br/><font color='#cc0000'>%1$d</font>出現時不要按下，<br/><font color='#cc0000'>%2$d</font>出現時，<br/>請您以最快的速度按下<br/><br/>正式開始前，會有小段練習<br/><br/><font color='#cc0000'>%3$d</font>秒后進入練習",
                        numberOne, numberTwo, countDown);
            case GameInfo.VERSION_GAME_THREE:
                return String.format(Locale.TRADITIONAL_CHINESE,"除數字<font color='#cc0000'>%1$d</font>以外的數字出現時，<br/>請您以最快的速度按下<br/><br/>正式開始前，會有小段練習<br/><br/><font color='#cc0000'>%2$d</font>秒后進入練習",
                        numberOne, countDown);
            default:
                return "";
        }
    }

    private void startPractise(){
        hideAllView();

        mVClickArea.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

            }
        });
    }

    private int getRandomNumber(){
        return (int)(Math.random() * 10);
    }

    private int getRandomNumber(int number){
        int value = getRandomNumber();
        return value == number ? getRandomNumber(number) : value;
    }

    private void hideAllView(){
        ConstraintLayout rootView = ((ConstraintLayout)getWindow().getDecorView().findViewById(R.id.root));
        for (int i = 0; i < rootView.getChildCount(); i ++){
            if (rootView.getChildAt(i).getId() != R.id.click_area)
                rootView.getChildAt(i).setVisibility(View.GONE);
        }
    }

    public static void LAUNCH_GAME_ONE(Activity context, GameInfo gameInfo){
        Intent intent = new Intent(context, GameActivity.class);
        intent.putExtra(PARAM_GAME_INFO, gameInfo);
        context.startActivity(intent);
    }
}