package com.jyc.threegames.bean;

public class ControllerMessage<T> {

    private ControllerMessage(){}

    public static final int CODE_SUCCESS = 200;
    public static final int CODE_FAILED = 500;

    private String message = "";
    private int code;
    private T object;

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public int getCode() {
        return code;
    }

    public void setCode(int code) {
        this.code = code;
    }

    public T getObject() {
        return object;
    }

    public void setObject(T object) {
        this.object = object;
    }

    public boolean isSuccess(){
        return this.code == CODE_SUCCESS;
    }

    public static <T> ControllerMessage<T> getSimpleMessage(boolean success){
        return new Builder<T>().setSuccess(success).build();
    }

    public static <T> ControllerMessage<T> getSimpleMessage(boolean success, String message){
        return new Builder<T>().setSuccess(success).msg(message).build();
    }

    public static <T> ControllerMessage<T> getSimpleMessage(boolean success, String message, T value){
        return new Builder<T>().setSuccess(success).msg(message).obj(value).build();
    }

    public static class Builder<T>{

        private ControllerMessage<T> result;

        public Builder(){
            result = new ControllerMessage<T>();
        }

        public ControllerMessage<T> build(){
            return result;
        }

        public Builder<T> setSuccess(boolean success){
            result.setCode(success ? CODE_SUCCESS : CODE_FAILED);
            return this;
        }

        public Builder<T> success(){
            result.setCode(CODE_SUCCESS);
            return this;
        }

        public Builder<T> success(String msg){
            msg(msg);
            result.setCode(CODE_SUCCESS);
            return this;
        }

        public Builder<T> failed(){
            result.setCode(CODE_FAILED);
            return this;
        }

        public Builder<T> failed(String msg){
            msg(msg);
            result.setCode(CODE_FAILED);
            return this;
        }

        public Builder<T> msg(String msg){
            result.setMessage(msg);
            return this;
        }

        public Builder<T> obj(T object){
            result.setObject(object);
            return this;
        }
    }
}

