package com.jyc.threegames.net;

import com.jyc.threegames.App;

import java.io.File;
import java.util.concurrent.TimeUnit;

import okhttp3.Cache;
import okhttp3.Interceptor;
import okhttp3.OkHttpClient;

public class OkHttpClientHelper {
    private static OkHttpClient mOkHttpDefaultClient;

    private static File cacheDirectory = new File(App.app.getApplicationContext().getCacheDir().getAbsolutePath(), "HttpCache");
    private static Cache cache = new Cache(cacheDirectory, 10 * 1024 * 1024);

    private OkHttpClientHelper() {}

    public static OkHttpClient getDefaultOkHttpClient(Interceptor... interceptors) {

        if (null == mOkHttpDefaultClient) {
            synchronized (OkHttpClientHelper.class){
                if (null == mOkHttpDefaultClient){
                    OkHttpClient.Builder builder = getBuilder();

                    for (Interceptor interceptor : interceptors){
                        builder.addInterceptor(interceptor);
                    }
                    mOkHttpDefaultClient = builder.build();
                }
            }
        }
        return mOkHttpDefaultClient;
    }

    public static OkHttpClient.Builder getBuilder(){
        OkHttpClient.Builder builder = new OkHttpClient.Builder();
        builder.connectTimeout(20, TimeUnit.SECONDS);
        builder.writeTimeout(20, TimeUnit.SECONDS);
        builder.readTimeout(20, TimeUnit.SECONDS);
        builder.sslSocketFactory(HttpsTrustManager.createSSLSocketFactory(), new HttpsTrustManager());
        builder.hostnameVerifier(new HttpsTrustManager.TrustAllHostnameVerifier());
        builder.cache(cache);
        return builder;
    }
}
