package com.jyc.threegames.controller;


import com.jyc.threegames.bean.ControllerMessage;
import com.jyc.threegames.net.ResData;

import io.reactivex.Observable;
import io.reactivex.ObservableEmitter;
import io.reactivex.ObservableOnSubscribe;
import io.reactivex.annotations.NonNull;
import io.reactivex.functions.Function;

public abstract class BaseController {
    Observable<ControllerMessage<Object>> getErrorObservable(final String message){
        return Observable.create(new ObservableOnSubscribe<ControllerMessage<Object>>() {
            @Override
            public void subscribe(@NonNull ObservableEmitter<ControllerMessage<Object>> e) throws Exception {
                if(!e.isDisposed()){
                    e.onError(new RuntimeException(message));
                    e.onComplete();
                }
            }
        });
    }

    <T>Observable<T> getSimpleSuccessObservable(final T obj){
        return Observable.create(new ObservableOnSubscribe<T>() {
            @Override
            public void subscribe(@NonNull ObservableEmitter<T> e) throws Exception {
                if (e.isDisposed())
                    return;
                e.onNext(obj);
                e.onComplete();
            }
        });
    }

    <T>Observable<T> getSimpleErrorObservable(final String message){
        return Observable.create(new ObservableOnSubscribe<T>() {
            @Override
            public void subscribe(@NonNull ObservableEmitter<T> e) throws Exception {
                if (e.isDisposed())
                    return;
                e.onError(new RuntimeException(message));
                e.onComplete();
            }
        });
    }

    Observable<ControllerMessage<Object>> getSimpleSuccessObservable(String message){
        return getSimpleSuccessObservable(ControllerMessage.getSimpleMessage(true, message));
    }

    Observable<ControllerMessage<Object>> getSimpleFailedObservable(String paramString)
    {
        return getSimpleSuccessObservable(ControllerMessage.getSimpleMessage(false, paramString));
    }

    protected class SimpleDataHandleFunction<T extends ResData, R extends ControllerMessage> implements Function<T, R>{
        @Override
        public R apply(T o) throws Exception {
            return (R) ControllerMessage.getSimpleMessage(o.isRequestSuccess(), o.message, o.data);
        }
    }
}
